#ifndef INCLUDE_COVIT_H
#define INCLUDE_COVIT_H

#include <cstdlib>
#include <stdio.h>
#include <iostream>
#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

//#include "rtp_api.h"
//#include "rtp_highlevel.h"

#include "interfaces.h"
#include "unicapVidCap.h"
//#include "mediabuffer.h"


using namespace std;

class CMediaBuffer
{
private	:
	char *dataBuffer;
	
	int tsinc_msec;	
	int buffer_size;
	int iframe;
	
public :
	int bufferCount;
	
	CMediaBuffer *ObjNextBuffer;
	CMediaBuffer *ObjPrevBuffer;	
	CMediaBuffer(int x)
	{
		bufferCount = x;
		dataBuffer = NULL;
		ObjNextBuffer = NULL;
		ObjPrevBuffer = NULL;
		iframe = 0;
	};
	
	~CMediaBuffer()
	{
		bufferCount = 0;
		freeBuffer();
		dataBuffer = NULL;
		ObjNextBuffer = NULL;
		ObjPrevBuffer = NULL;
	};

	char *getBuffer(int *size,int *timestampInc)
	{
		*size = buffer_size; 
		
		if(timestampInc != NULL)
			*timestampInc = tsinc_msec; 
		return dataBuffer; 
	};

	void freeBuffer()
	{
		free(dataBuffer);
		dataBuffer = NULL;
	};	
	int setBuffer(int size, char *buf, int i_frame, int ts_msec)
	{
		dataBuffer = (char *)malloc(size * sizeof(char));
		if(dataBuffer == NULL)
		{
			cout << "Cannot allocate Memory " << endl;
			return -1;		
		}
		memcpy(dataBuffer,buf, size);
		buffer_size = size;
		tsinc_msec = ts_msec;
		iframe = i_frame;
		return 0;

	};
	void setNextBuffer(CMediaBuffer *nxtbuf)
	{
		ObjNextBuffer = nxtbuf;	
	};	
	void setPrevBuffer(CMediaBuffer *prevbuf)
	{
		ObjPrevBuffer = prevbuf;
		
	};
};

class CSenderThreadParams
{
public:
	int threadid;
	pthread_cond_t *MediaBuffer_Empty_cond[30];
	pthread_cond_t *MediaBuffer_Full_cond[30];
	pthread_mutex_t *MediaBuffer_mutex[30];
	CMediaBuffer *MediaBuffer;
	char host[256];
	int port;
	int ttl;	

	pthread_mutex_t *thr_mutex; 
	pthread_cond_t  *thr_cond; 


	pthread_mutex_t *sendstop_cond_mutex; // = PTHREAD_MUTEX_INITIALIZER;
	pthread_cond_t  *sendstop_cond; //  = PTHREAD_COND_INITIALIZER;

	pthread_mutex_t *sendstatus_mutex; // = PTHREAD_MUTEX_INITIALIZER;
	int *send_quit;

};

class CCapThreadParams
{
	public:
	pthread_cond_t *MediaBuffer_Empty_cond[30];
	pthread_cond_t *MediaBuffer_Full_cond[30];
	pthread_mutex_t *MediaBuffer_mutex[30];


	pthread_cond_t *m_delMediaBuffer_Empty_cond;
	pthread_cond_t *m_delMediaBuffer_Full_cond;
	pthread_mutex_t *m_delMediaBuffer_mutex;

	CMediaBuffer *MediaBuffer;

	pthread_mutex_t *thr_mutex; 
	pthread_cond_t  *thr_cond; 
	pthread_mutex_t *capstatus_mutex;

	pthread_mutex_t *thrstop_mutex;
	pthread_cond_t *thrstop_cond;
	int *stopCapStatus;		 
	int threadid;


};

class CRecvThreadParams
{
public:
	//CMediaBuffer *MediaBuffer;
	char host[256];
	int port;
	int ttl;	
		
	int threadid;
	pthread_mutex_t *thr_mutex; 
	pthread_cond_t  *thr_cond; 

	pthread_mutex_t *recvstop_mutex; 
	pthread_cond_t  *recvstop_cond; 
	int *recv_quit;

};


class CVideoEnc : public IVideoEnc
{
public	:
	CVideoEnc(){};
	virtual ~CVideoEnc(){};
	virtual void encode_frame();
	virtual void convert_img_format();
};


class CVideoCapture : public IVideoCapture
{
protected :
	CUnicapVidCap *ObjVidCap;
	SDL_Rect rect;
	SDL_Surface *m_screen;

	int m_stopStatusBase;
	pthread_mutex_t *m_capstatus_mutex;

	pthread_mutex_t *m_thrstop_mutex;
	pthread_cond_t *m_thrstop_cond;	

	pthread_cond_t *m_MediaBuffer_Empty_cond[30];
	pthread_cond_t *m_MediaBuffer_Full_cond[30];
	pthread_mutex_t *m_MediaBuffer_mutex[30];

	pthread_cond_t *m_delMediaBuffer_Empty_cond;
	pthread_cond_t *m_delMediaBuffer_Full_cond;
	pthread_mutex_t *m_delMediaBuffer_mutex;


	int m_encode_bitrate;
	int m_fps;
	int m_gopSize;
	CodecID m_encode_fmt;

public	:
	int m_threadid;
	CMediaBuffer *m_MediaBuffer; 
	CVideoEnc *VideoEnc;
	
	CVideoCapture(int threadid);
	
	virtual ~CVideoCapture();
	
	int getStopStatus();
	int setStopStatus();
	
	virtual void SetSettings(int device_no);	
	void Process_Raw_Video(int buffers_ready,int PixFormat, int bufsize,int width, int height,unsigned char *data, int tsinc_usec);	
	
	virtual void getVideoFormat();
	virtual void setVideoFormat();
	virtual void getVideoDeviceProp();
	virtual void setMediaBuffer(CCapThreadParams*);
	virtual void start_capture(CCapThreadParams *cparam);
	virtual int stop_capture();
	
};

class CCameraProperties
{


};

class CVideoFormat
{


};



#endif
