#include <iostream>
#include <list>
#include <cstdlib>   
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>
#include <pthread.h>

#ifdef __unix
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <sys/time.h>  /* struct timeval, gettimeofday() */
#include <sys/param.h> /* Might declare MAXHOSTNAMELEN */
#include <netdb.h>     /* Might declare MAXHOSTNAMELEN */
#include <unistd.h>    /* Might declare gethostname() */
#endif

#ifdef WIN32
#include <winsock2.h>
#define MAXHOSTNAMELEN  50
extern void gettimeofday(struct timeval *, void *);
#endif
#include "covit.h"
#include "common_defines.h"
extern list<capNode> capList;

extern pthread_mutex_t libavcodec_mutex;

extern pthread_mutex_t sendthr_cond_mutex;
extern pthread_cond_t  sendthr_cond;
int send_quit[20];

extern pthread_mutex_t sendstatus_mutex;
extern pthread_mutex_t sendstop_cond_mutex;
extern pthread_cond_t  sendstop_cond;

		
extern "C" {
#include "rtp_api.h"
#include "rtp_highlevel.h"
}
#include "covit.h"
#define PAYLOAD_TYPE_MULAW_8 0

/* Convert a struct timeval to a double */
#define tv2dbl(tv) ((tv).tv_sec + (tv).tv_usec / 1000000.0)

using namespace std;

/* Convert a double to a struct timeval */
static struct timeval dbl2tv(double d) 
{
  struct timeval tv;

  tv.tv_sec = (long) d;
  tv.tv_usec = (long) ((d - (long) d) * 1000000.0);
  
  return tv;
}

/* Functions that implement the RTP Scheduler */

/* We maintain a simple queue of events. */
/* If you're not using the library in a simple command-line tool like this,
   you will probably need to tie in to your UI library's event queue
   somehow, instead of using this simple approach.*/
struct evt_queue_elt {
  context cid;
  rtp_opaque_t event_opaque;
  double event_time;
  struct evt_queue_elt *next;
};

static struct evt_queue_elt* evt_queue = NULL;

static void insert_in_evt_queue(struct evt_queue_elt *elt)
{
  if (evt_queue == NULL || elt->event_time < evt_queue->event_time) {
    elt->next = evt_queue;
    evt_queue = elt;
  }
  else {
    struct evt_queue_elt *s = evt_queue;
    while (s != NULL) {
      if (s->next == NULL || elt->event_time < s->next->event_time) {
	elt->next = s->next;
	s->next = elt;
	break;
      }
      s = s->next;
    }
  }
}

/* The library calls this function when it needs to schedule an event.
 *
 * Note that if we were running more than one stream at once we'd need
 * some way of corrolating the event to run with the context ID this
 * function gets passed.
 */
void RTPSchedule(context cid, rtp_opaque_t opaque, struct timeval *tp)
{
  struct evt_queue_elt *elt;

  elt = (struct evt_queue_elt *) malloc(sizeof(struct evt_queue_elt));
  if (elt == NULL)
    return;

  elt->cid = cid;
  elt->event_opaque = opaque;
  elt->event_time = tv2dbl(*tp);
  
  insert_in_evt_queue(elt);

 // free(elt);
  return;
}

/* Parse an addr/port/ttl string. */
static int hpt(char *h, u_int16 *port, unsigned char *ttl)
{
  char *s;
  char temp[40];	
  /* first */
  s = strchr(h, '/');
  strncpy(temp, h, s-h);
  temp[s-h+1] = '\0';		
  //strcpy(h, temp);	

 // cout << temp << " " << s <<endl;	
  if (!s) {
    return -1;
  }
  else {
    *s = '\0';
    *port = atoi(s+1);
    s = strchr(s+1, '/');
    if (s && ttl) {
      *ttl = atoi(s+1);
    }
  }
  return 0;
} /* hpt */


/* Set up an RTP Library connection.
 * Return the maximum file descriptor number used, or -1 on error.
 * remote should contain the address to connect to in
 * host/port[/ttl] format.  (If ttl is not specified, '1' is used.)
 * cid will contain the context ID of the RTP session on sucessful return.
 * sock will contain the file descriptors of the RTP and RTCP ports.
 *
 * BUG: on failure, we should tear down the partially-completed
 * connections. */
static int setup_connection(CSenderThreadParams *TParam, context *cid, int sock[2])
{
  /* Error values that the RTP library returns. */
  rtperror err;
  /* The port of the remote address */
  u_int16 port = (u_int16)(TParam->port);
  /* The time-to-live value to set if we're talking to a multicast
   * address. */
  unsigned char ttl = 1;
  /* The sockaddr that hpt returns to us.  Note that we don't actually
   * pass this value to the library; it's just a convenient structure. */
  /* The type the library uses to keep track of sockets. */
  socktype sockt;
  /* The highest file descriptor we've dealt with. */
  int nfds = 0;

  /* Some values for creating the CNAME (see below). */
  char *username;
  char hostname[MAXHOSTNAMELEN];
  char cname[MAXHOSTNAMELEN+32];
  struct timeval tp;
  person uid, conid;
#ifdef WIN32
  WORD wVersionRequested;
  WSADATA wsaData;
  int err2;
 
  wVersionRequested = MAKEWORD( 2, 0 );
 
  err2 = WSAStartup( wVersionRequested, &wsaData );
  if ( err2 != 0 ) {
	  /* Tell the user that we couldn't find a usable */
	  /* WinSock DLL.                                  */
	  return(-1);
  }
#endif
  /* Translate host/port[/ttl] structure into sockaddr_in and ttl value. */
 // if (hpt(remote, &port, &ttl) < 0) {
  //  fprintf(stderr, "%s: bad address\n", remote);
  //  return -1;
 // }

  /* Create the RTP session -- allocate data structures.
   * This doesn't open any sockets. */
  err = RTPCreate(cid);
  if (err != RTP_OK) {
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  } 
	RTPSessionSetRTPStampRate(*cid, 100,
	10);
  char *remote = TParam->host;
  
  ttl = TParam->ttl;			
  //cout << "Remote " << remote << endl;	
  /* Set the RTP Session's host address to send to. */
  err = RTPSessionAddSendAddr(*cid, remote, port, ttl);
  if (err != RTP_OK) {
 //   cout << remote << " " << port << " " << ttl << endl;  	
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  }

  
  /* Set the RTP Session's local receive address */
  /* We set this to the dest for the following reason:
     - It's necessary if the dest is a multicast group, so we join the group
     - It's okay if the dest is unicast, since setting a dest which isn't a
       local address is ignored. */
 err = RTPSessionSetReceiveAddr(*cid, NULL, port);

cout << remote << " " << TParam->host << endl;
  /* Set up our CNAME (canonical name): should be of the form user@host */
  username = getenv("USER");
  if (gethostname(hostname, MAXHOSTNAMELEN) < 0) {
    perror("gethostname");
    return -1;
  }
  if (username) { sprintf(cname, "%s@%s", username, hostname); }
  else { strcpy(cname, hostname); }

  /* Member 0 is the local member (us) */
  err = RTPMemberInfoSetSDES(*cid, 0, RTP_MI_CNAME, cname);
  if (err != RTP_OK) {
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  }

  /* Set up our NAME (standard display name) */
  err = RTPMemberInfoSetSDES(*cid, 0, RTP_MI_NAME, "RTP Example Server");
  if (err != RTP_OK) {
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  }
//cout << remote << " 2 " << TParam->host << endl;
  /* Add a single contributor */

  gettimeofday(&tp, NULL);
  srand(tp.tv_usec);
  conid = rand();
  err = RTPSessionAddToContributorList(*cid, (int) conid);
  if (err) {
    fprintf(stderr, "%s %s\n", RTPStrError(err), RTPDebugStr());
    return -1;
  }
  err = RTPSessionGetUniqueIDForCSRC(*cid,conid,&uid);
  if (err) {
    fprintf(stderr, "%s %s\n", RTPStrError(err), RTPDebugStr());
    return -1;
  }
  sprintf(cname, "Contributor %d", (int) conid);
  err = RTPMemberInfoSetSDES(*cid, uid, RTP_MI_CNAME, cname);
  if (err) {
    fprintf(stderr, "%s %s\n", RTPStrError(err), RTPDebugStr());
    return -1;
  }
  err = RTPMemberInfoSetSDES(*cid, uid, RTP_MI_NAME, cname);
  if (err) {
    fprintf(stderr, "%s %s\n", RTPStrError(err), RTPDebugStr());
    return -1;
  }

RTPSessionSetBandwidth(*cid, 400,0.05);
  /* Open the connection.  We're now live on the network. */
  err = RTPOpenConnection(*cid);
  if (err != RTP_OK) {
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  }

  /* Get the socket that RTP data is transmitted on, so we can select() on
   * it. */
  /* Note that the value passed to it is a socktype *, not an int *. */
  err = RTPSessionGetRTPSocket(*cid, &sockt);
  if (err != RTP_OK) {
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  }
  sock[0] = sockt;
  nfds = 0;
#ifdef __unix
  if (nfds < sockt) nfds = sockt;
#endif


  /* Get the socket that RTCP control information is transmitted on, so we
     can select() on it. */
  err = RTPSessionGetRTCPSocket(*cid, &sockt);
  if (err != RTP_OK) {
    fprintf(stderr, "%s\n", RTPStrError(err));
    return -1;
  }
  sock[1] = sockt;
#ifdef __unix
  if (nfds < sockt) nfds = sockt;
#endif

  return nfds;
}

/*
Name: sendrtp
Input parameters: 
void *param: Parameters passed to the receiving thread
Output: NONE 
Description:
Function for the sending Thread
*/
void *sendrtp(void *param)
{
	cout << "sendrtp" <<endl;
	CSenderThreadParams *sparam = (CSenderThreadParams *)param;	

	/*set the mediabuffer variables that are passed as parameter to the thread */	
	pthread_cond_t *zMediaBuffer_Empty_cond[30];
	pthread_cond_t *zMediaBuffer_Full_cond[30];
	pthread_mutex_t *zMediaBuffer_mutex[30];
	for(int i = 0; i < 30; i++)
	{
		zMediaBuffer_Empty_cond[i] = sparam->MediaBuffer_Empty_cond[i] ;
		zMediaBuffer_Full_cond[i] = sparam->MediaBuffer_Full_cond[i];
		zMediaBuffer_mutex[i] = sparam->MediaBuffer_mutex[i] ;
	}
	
	CMediaBuffer *MediaBuffer = sparam-> MediaBuffer;
	int id = sparam->threadid;
	//pthread_mutex_t *zthr_mutex = sparam->thr_mutex; 
	//pthread_cond_t  *zthr_cond = sparam->thr_cond; 

	pthread_mutex_t *zsendstop_cond_mutex = sparam->sendstop_cond_mutex; // = PTHREAD_MUTEX_INITIALIZER;
	pthread_cond_t  *zsendstop_cond = sparam->sendstop_cond; //  = PTHREAD_COND_INITIALIZER;

	pthread_mutex_t *zsendstatus_mutex = sparam->sendstatus_mutex; // = PTHREAD_MUTEX_INITIALIZER;
	int *send_quit = sparam->send_quit;
	
	CodecID encode_fmt;
	int encode_width;
	int encode_height;
	
	list<capNode>::iterator p;
	for(p = capList.begin(); p!= capList.end(); ++p)
	{
		if((*p).id == sparam->threadid)
		{
			encode_width = (*p).encodeFmtNode.encode_width;
			encode_height = (*p).encodeFmtNode.encode_height;
			encode_fmt = 	(*p).encodeFmtNode.encode_fmt;		
						
			break;
		}
	}
 

	do{
		pthread_mutex_lock( zsendstatus_mutex );		 	
			send_quit[id] = 0;
		pthread_mutex_unlock( zsendstatus_mutex );		 	
			  
		CMediaBuffer *rtpMediaBuffer = MediaBuffer;//TParam->MediaBuffer;

		/* The context id the RTP library gives us to keep track of our RTP
		stream. */
		context cid;
		int max_bufsize = 512;
		/* The two socket fds (RTP and RTCP) we need to select on. */
		socktype sock[2];
		/* The highest file descriptor we've seen, of those we need to select
		   on. */
		int nfds;

		/* Maximum size of a UDP packet. */
		#define RECEIVE_BUFFER_SIZE 65536
		/* The buffer incoming data accumulates in.  We don't actually care
		* about the contents of this data, but we must call RTPReceive() so our
		* sender reports and receiver reports are correct. */
		
		char fr_buffer[RECEIVE_BUFFER_SIZE];
		/* Some values for keeping track of when to send packets. */
		struct timeval start_time_tv;
		double start_time;
	
		/* The RTP marker bit; initially, it's set to 0. */
		int8 marker = 0;

		/* Error values that the RTP library returns. */
		rtperror err;
		if ((nfds = setup_connection(sparam, &cid, sock)) < 0) {
			cout << "reached" << endl;    
			exit(1);
		}
	

		gettimeofday(&start_time_tv, NULL);
		start_time = tv2dbl(start_time_tv);
		int newframe = 0;
		pthread_mutex_lock( zsendstatus_mutex );		 	
		while(!(*send_quit))	
		{
			pthread_mutex_unlock( zsendstatus_mutex );   
			newframe = 1;	
			int out_size;
			/* timestamp increment */
			int ts;
			pthread_mutex_lock(zMediaBuffer_mutex[rtpMediaBuffer->bufferCount]);
			unsigned char *databuf = (unsigned char *)rtpMediaBuffer->getBuffer(&out_size,&ts);
			if(databuf == NULL)
			{
				//cout << "sendRTP Waiting " << &rtpMediaBuffer << endl;
				pthread_cond_wait(zMediaBuffer_Full_cond[rtpMediaBuffer->bufferCount], zMediaBuffer_mutex[rtpMediaBuffer->bufferCount]);
				//cout << "sendRTP Waiting Fin" << endl;
				databuf = (unsigned char *)rtpMediaBuffer->getBuffer(&out_size,&ts);
			}
			pthread_mutex_unlock(zMediaBuffer_mutex[rtpMediaBuffer->bufferCount]);	
			
	
			int toffset = 0;	
			while(max_bufsize < out_size)     	
			{
				if(newframe == 1)
				{	
					/* Send the data we just read to the network.*/
					err = RTPSend(cid, 100000*ts/1000, marker, 100,(int8 *) databuf,
					  max_bufsize);
				}
				else
				{
					/* Send the data we just read to the network.*/
					err = RTPSend(cid, 0, marker, 100,(int8 *) databuf,
					  max_bufsize);
				}

				memcpy(fr_buffer + toffset, databuf, max_bufsize);
				newframe = 0;
				//marker = 0;
				toffset += max_bufsize;
				databuf += max_bufsize;
				out_size -= max_bufsize;

			}
	
			// Set the marker bit to indicate the last packet of a video frame
			marker = 1;	
			if(newframe == 1)
			{	

				err = RTPSend(cid, 100000*ts/1000, marker, 100,(int8 *) databuf,
					  out_size);
				memcpy(fr_buffer + toffset, databuf, out_size);

				//marker = 0;
				toffset += out_size;

			}
			else
			{

				err = RTPSend(cid, 0, marker, 100,(int8 *) databuf,
					  out_size);
				memcpy(fr_buffer + toffset, databuf, out_size);
				toffset += out_size;

			}

	// for Displaying the video in the sender side
	#if 0
			{
				static AVCodec *dec;
				static AVCodecContext *decctx = NULL;
				AVFrame *decpict;
				int i, decout_size, decout_buf_size;
				uint8_t *decout_buf, *decpicture_buf;
			
				char buf[1024];	
				int got_pict;
				
				SDL_LockYUVOverlay(bmp);
				AVPicture pict;
				pict.data[0] = bmp->pixels[0];
				pict.data[1] = bmp->pixels[2];//bmp->pixels[2];
				pict.data[2] = bmp->pixels[1];

				pict.linesize[0] = bmp->pitches[0];
				pict.linesize[1] = bmp->pitches[2];
				pict.linesize[2] = bmp->pitches[1];

	
		
				static int first = 0;			
				if(!first)
				{
					pthread_mutex_lock(&libavcodec_mutex);			
					dec = avcodec_find_decoder(encode_fmt);//CODEC_ID_MPEG1VIDEO);//CODEC_ID_MPEG1VIDEO);//CODEC_ID_MPEG1VIDEO);
					if(!dec)
					{
						cout << "Codec Not Found" << endl;
					} 
					pthread_mutex_unlock(&libavcodec_mutex);			
					decctx = avcodec_alloc_context();
					//decpict = avcodec_alloc_frame();
	
					decctx->width = encode_width;
					decctx->height = encode_height;
					decctx->pix_fmt = PIX_FMT_YUV420P;
	
					pthread_mutex_lock(&libavcodec_mutex);
					if(avcodec_open(decctx, dec) < 0)
					{
						cout << "Dec Could not open Codec" << endl;
						exit(1);		
					}
					pthread_mutex_unlock(&libavcodec_mutex);
					first = 1;
				}
	
				decpict = avcodec_alloc_frame();
				//memcpy(inbuf_ptr, out_buf, out_size);
		
	#if 0
				else
				{
					rtpMediaBuffer->freeBuffer();
					//pthread_cond_signal(&MediaBuffer_Empty_cond[rtpMediaBuffer->bufferCount]);		
	    				pthread_mutex_unlock(&MediaBuffer_mutex[rtpMediaBuffer->bufferCount]);
					rtpMediaBuffer = rtpMediaBuffer->ObjNextBuffer;
	    	
					continue;
				}
	#endif

				int len = avcodec_decode_video(decctx, decpict/*(AVFrame *)&pict*/, &got_pict, (uint8_t *)fr_buffer, toffset);
		//		cout << "decode : " << len  << " " << toffset << " " << got_pict <<endl;
				//pthread_mutex_unlock(&libavcodec_mutex);

	
				// Convert the image into YUV format that SDL uses
				img_convert(&pict, PIX_FMT_YUV420P,(AVPicture *)decpict, decctx->pix_fmt , decctx->width, decctx->height);
		
				SDL_UnlockYUVOverlay(bmp);
				rect.x = 0;
				rect.y = 0;

				rect.w = width;
				rect.h = height;
				SDL_DisplayYUVOverlay(bmp, &rect);
				pthread_mutex_lock(&libavcodec_mutex);			
				//avcodec_close(decctx);			
				pthread_mutex_unlock(&libavcodec_mutex);			
				//av_free(decctx);
				av_free(decpict);
			
			}
	#endif

				pthread_mutex_lock(zMediaBuffer_mutex[rtpMediaBuffer->bufferCount]);	    
				rtpMediaBuffer->freeBuffer();
				pthread_cond_signal(zMediaBuffer_Empty_cond[rtpMediaBuffer->bufferCount]);		
				pthread_mutex_unlock(zMediaBuffer_mutex[rtpMediaBuffer->bufferCount]);
				rtpMediaBuffer = rtpMediaBuffer->ObjNextBuffer;
				//cout << "Pkt Sent "<< out_size << endl;
				//quit = 1;		
				if (err != RTP_OK) {
					fprintf(stderr, "%s\n", RTPStrError(err));
					exit(1);
				}
				/* The marker bit doesn't get set for any packet but the first. */
				marker = 0;
				pthread_mutex_lock( zsendstatus_mutex );		 	

			}
		pthread_mutex_unlock( zsendstatus_mutex );		 	
		
		/* We're done sending now, so close the connection. */
		/* This sends the RTCP BYE packet and closes our sockets. */
		if ((err = RTPCloseConnection(cid, "Goodbye!")) != RTP_OK) {
			fprintf(stderr, "%s\n", RTPStrError(err));
		}

		/* De-allocate all the data structures reserved for the connection.
		* (This would also close the connection, as well, if we hadn't just done
		* it above.) */
		if ((err = RTPDestroy(cid)) != RTP_OK) {
			fprintf(stderr, "%s\n", RTPStrError(err));
		}
	//enable to free resources needed for displaying video window at the sender
	#if 0
		SDL_FreeYUVOverlay(bmp);
		SDL_FreeSurface(m_screen);
		//SDL_Quit();
		m_screen = NULL;		
	#endif
		pthread_mutex_lock( zsendstop_cond_mutex );
		pthread_cond_signal( zsendstop_cond );
		pthread_mutex_unlock( zsendstop_cond_mutex );

	}while(0);

	free(sparam);		
	pthread_exit(NULL);	
	cout << "leaving" << endl;	
	return 0;
}


