/* A simple synchronous XML-RPC client written in C, as an example of
   an Xmlrpc-c client.  This invokes the sample.add procedure that the
   Xmlrpc-c example server.c server provides.  I.e. it adds to numbers
   together, the hard way.
 g++ `xmlrpc-c-config --cflags` xmlrpc_sample_add_client.cpp -o client `xmlrpc-c-config --libs` -lxmlrpc_client -pthread

*/

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <xmlrpc-c/base.h>
#include <xmlrpc-c/client.h>

#include "config.h"  /* information about this build environment */
using namespace std;
#define NAME "Xmlrpc-c Test Client"
#define VERSION "1.0"

static void 
die_if_fault_occurred (xmlrpc_env *env) {
    if (env->fault_occurred) {
        fprintf(stderr, "XML-RPC Fault: %s (%d)\n",
                env->fault_string, env->fault_code);
        exit(1);
    }
}



int 
main(int           const argc, 
     const char ** const argv ATTR_UNUSED) {

    xmlrpc_env env;
    xmlrpc_value *result;
    xmlrpc_int32 sum;
    char *name;
    char methodN[200];	
    
	char serverUrl[200];
	strcpy(serverUrl,argv[1]);
    char methodName[200];// = "QueryInterface";//"getCameraProps";//"sample.add";

    if (argc != 2) {
        fprintf(stderr, "This program has 1 arguments- ./client http://localhost:8080/RPC2\n");
        exit(1);
    }
    	
    xmlrpc_env_init(&env);	
/* Start up our XML-RPC client library. */
    xmlrpc_client_init2(&env, XMLRPC_CLIENT_NO_FLAGS, NAME, VERSION, NULL, 0);
    die_if_fault_occurred(&env);

    do {	
	    cout << "Enter Method Name:";	
	    cin >> methodName;
	
    	    if(strcmp("StartStreamingRTP", methodName) == 0) 
	    {
		      char ip[255], port[255]; int portc;
		      char id[255];		
		      xmlrpc_int32 streamId; 
		      cout << "Enter StreamID: ";
		      cin >> id;
		      cout << "Enter IP address: ";
		      cin >> ip;
		      cout << "Enter Port: ";
		      cin >> port;
      		
		      xmlrpc_value *arr = xmlrpc_array_new(&env);
		      xmlrpc_value *v1 = xmlrpc_build_value(&env, "s", &ip);
		      xmlrpc_value *v2 = xmlrpc_build_value(&env, "s", &port);
		      xmlrpc_value *v3 = xmlrpc_build_value(&env, "s", &id);				
		      xmlrpc_array_append_item (&env,arr,v1);
		      xmlrpc_array_append_item (&env,arr,v2);
		      xmlrpc_array_append_item (&env,arr,v3);

		      result = xmlrpc_client_call(&env, serverUrl, methodName,
		                              "(A)", arr);
	    	      die_if_fault_occurred(&env);
		      xmlrpc_read_int(&env, result, &streamId);
		      die_if_fault_occurred(&env);
		      printf("The streamId is %d\n", streamId);
		  
	    }
	    else if(strcmp("StartReceivingRTP", methodName) == 0)	
	    {
		        char ip[255], port[255]; int portc; 
		        char id[255];		
		        xmlrpc_int32 streamId;
		        cout << "Enter StreamID: ";
		        cin >> id;
		        cout << "Enter IP address: ";
		        cin >> ip;
		        cout << "Enter Port: ";
		        cin >> port;
        				

		        xmlrpc_value *arr = xmlrpc_array_new(&env);
		        xmlrpc_value *v1 = xmlrpc_build_value(&env, "s", &ip);
		        xmlrpc_value *v2 = xmlrpc_build_value(&env, "s", &port);		
		        xmlrpc_value *v3 = xmlrpc_build_value(&env, "s", &id);		
		        xmlrpc_array_append_item (&env,arr,v1);
		        xmlrpc_array_append_item (&env,arr,v2);
		        xmlrpc_array_append_item (&env,arr,v3);

		        result = xmlrpc_client_call(&env, serverUrl, methodName,
		                                "(A)", arr);
	    	        die_if_fault_occurred(&env);
			        xmlrpc_read_int(&env, result, &streamId);
		        die_if_fault_occurred(&env);
		        printf("The streamId is %d\n", streamId);
      }	
	    else if(strcmp("setEncodingFormats", methodName) == 0)
	    {
		    char codec[255], width[255], height[255], bitrate[255]; int portc; 
		    char id[255];	
		    cout << "Enter StreamID: ";
		    cin >> id;
		    cout << "Enter Codec Tye [MPEG4 MPEG2 MPEG1]: ";
		    cin >> codec;
		    cout << "Enter Width Height Bitrate: ";
		    cin >> width;
		    cin >> height;
		    cin >> bitrate;
    		
    		
		    xmlrpc_value *arr = xmlrpc_array_new(&env);
		    xmlrpc_value *v1 = xmlrpc_build_value(&env, "s", &codec);
		    xmlrpc_value *v2 = xmlrpc_build_value(&env, "s", &width);
		    xmlrpc_value *v3 = xmlrpc_build_value(&env, "s", &height);		
		    xmlrpc_value *v4 = xmlrpc_build_value(&env, "s", &bitrate);				
		    xmlrpc_value *v5 = xmlrpc_build_value(&env, "s", &id);

		    xmlrpc_array_append_item (&env,arr,v1);
		    xmlrpc_array_append_item (&env,arr,v2);
		    xmlrpc_array_append_item (&env,arr,v3);
		    xmlrpc_array_append_item (&env,arr,v4);
		    xmlrpc_array_append_item (&env,arr,v5);

		    result = xmlrpc_client_call(&env, serverUrl, methodName,
		                            "(A)", arr);
	    	    die_if_fault_occurred(&env);
    	    

	    }
	    else if(strcmp("setDecodingFormat", methodName) == 0)
	    {
		char codec[255], width[255], height[255]; int portc;
		char id[255];	
		cout << "Enter StreamID: ";
		cin >> id;
		cout << "Enter Codec Tye [MPEG4 MPEG2 MPEG1]: ";
		cin >> codec;
		cout << "Enter Width Height: ";
		cin >> width;
		cin >> height;
		
		
		
		xmlrpc_value *arr = xmlrpc_array_new(&env);
		xmlrpc_value *v1 = xmlrpc_build_value(&env, "s", &codec);
		xmlrpc_value *v2 = xmlrpc_build_value(&env, "s", &width);
		xmlrpc_value *v3 = xmlrpc_build_value(&env, "s", &height);		
		xmlrpc_value *v4 = xmlrpc_build_value(&env, "s", &id);

		xmlrpc_array_append_item (&env,arr,v1);
		xmlrpc_array_append_item (&env,arr,v2);
		xmlrpc_array_append_item (&env,arr,v3);
		xmlrpc_array_append_item (&env,arr,v4);

		result = xmlrpc_client_call(&env, serverUrl, methodName,
		                        "(A)", arr);
	    	die_if_fault_occurred(&env);
	    

	    }
	 	else if(strcmp(methodName, "StopSend") == 0)
		{
		   int streamId;	
		   xmlrpc_int32 id;
		   cout << "Enter Stream Id: ";
		   cin >> streamId;		
		   id = (xmlrpc_int32)(streamId);	
			/* Make the remote procedure call */
		    result = xmlrpc_client_call(&env, serverUrl, methodName,
				                "(i)", id);
		    die_if_fault_occurred(&env);
		}	
		else if(strcmp(methodName, "StopRecv") == 0)
		{
		    
		   int streamId;	
		   xmlrpc_int32 id;
		   cout << "Enter Stream Id: ";
		   cin >> streamId;		
		   id = (xmlrpc_int32)(streamId);	
			/* Make the remote procedure call */
		    result = xmlrpc_client_call(&env, serverUrl, methodName,
				                "(i)", id);
		    die_if_fault_occurred(&env);
		}	
		else if(strcmp(methodName, "CreateRecvStream") == 0)
		{
		    
		   int streamId;	
		   xmlrpc_int32 id;
		   cout << "Enter Stream Id: ";
		   cin >> streamId;		
		   id = (xmlrpc_int32)(streamId);	
			/* Make the remote procedure call */
		    result = xmlrpc_client_call(&env, serverUrl, methodName,
				                "(i)", id);
		    die_if_fault_occurred(&env);
		}
		else if(strcmp(methodName, "CreateSendStream") == 0)
		{
		    
		   int streamId;	
		   xmlrpc_int32 id;
		   cout << "Enter Stream Id: ";
		   cin >> streamId;		
		   id = (xmlrpc_int32)(streamId);	
			/* Make the remote procedure call */
		    result = xmlrpc_client_call(&env, serverUrl, methodName,
				                "(i)", id);
		    die_if_fault_occurred(&env);
		}
	    else if(strcmp(methodName, "setCameraProps") == 0)
		{
			int width, ht;
			printf("Enter width and height: ");
			cin >> width;
			cin >> ht;
			result = xmlrpc_client_call(&env, serverUrl, methodName,
				                "(ii)", (xmlrpc_int32) width, (xmlrpc_int32) ht );

			die_if_fault_occurred(&env);
		}				
	    else 
	    {			    		
	   // gets(&methodN); 	
	    /* Initialize our error-handling environment. */
		    

		    
		    printf("%s %s\n", serverUrl, methodName);

			/* Make the remote procedure call */
		    result = xmlrpc_client_call(&env, serverUrl, methodName,
				                "(i)", (xmlrpc_int32) 0);
		    die_if_fault_occurred(&env);
	    }		    
	    if(	strcmp(methodName,"QueryInterface") == 0)
	    {	
		    int no_items = xmlrpc_array_size(&env, result);
		    cout << no_items << endl; 
		    char *method[no_items];
		    for(int i = 0; i < no_items; i++)
		    {
			xmlrpc_value * ElementP;
			xmlrpc_int firstInt;

			xmlrpc_array_read_item(&env, result, i, &ElementP);

			xmlrpc_read_string(&env, ElementP, (const char **)&method[i]);
			printf("Supported Method[%d] is %s\n", i, method[i]);
			/* prints "First element is 5" */
			xmlrpc_DECREF(ElementP);	
		    }	
	
		    /* Get our sum and print it out. */
		   // xmlrpc_read_string(&env, result, (const char **)&name); //&sum //);
		    die_if_fault_occurred(&env);
		//    printf("The sum is %d\n", sum);
	    }    
	    /* Dispose of our result value. */
	    xmlrpc_DECREF(result);
	    
 	
	}while(strcmp(methodName,"Quit") != 0);	
    
	/* Clean up our error-handling environment. */
    	xmlrpc_env_clean(&env);


    /* Shutdown our XML-RPC client library. */
    xmlrpc_client_cleanup();
	
//    free(methodName);
 //   free(serverUrl);		
    return 0;
}

