/* A simple standalone XML-RPC server written in C. */


// g++ `xmlrpc-c-config --cflags` xmlrpc_sample_add_client.cpp -o client `xmlrpc-c-config --libs` -lxmlrpc_client -pthread
// g++ `xmlrpc-c-config --cflags` xmlrpc_servertest.cpp -o serv `xmlrpc-c-config --libs` -lxmlrpc_server_abyss -pthread

#include <stdlib.h>
#include <stdio.h>
#ifndef WIN32
#include <unistd.h>
#endif

#include <xmlrpc-c/base.h>
#include <xmlrpc-c/server.h>
#include <xmlrpc-c/server_abyss.h>
#include <sys/time.h>
#include <string.h>
#include <time.h>
#include "covit.h"
#include "config.h"  /* information about this build environment */


extern pthread_t recvthr, capthr, sendthr;
extern int starting(int state, char *ip_address, int port, int streamId);
extern int stopLocal(int state);
extern int stopSend(int state);
extern int stopRecv(int state);
extern int createSendStreamId(int id);
extern int createRecvStreamId(int id);
extern void init_all();

extern CVideoCapture VidCapturer;

/*
Name: CreateSendStream
Description: 
method of XML RPC server to create Send Stream with provided id 
*/
static xmlrpc_value *
CreateSendStream(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    xmlrpc_int32 state;
	
    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;

    int res = createSendStreamId((int)(state));
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

/*
Name: CreateRecvStream
Description: 
method of XML RPC server to create Recv Stream with provided id
*/
static xmlrpc_value *
CreateRecvStream(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    xmlrpc_int32 state;
	
    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;

    int res = createRecvStreamId((int)(state));
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

/*
Name: getCameraProps
Description: 
method of XML RPC server to get Camera properties (dummy Method, not implemented)
*/
static xmlrpc_value *
getCameraProps(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    xmlrpc_int32 state;
	int res = 0;
    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;

     
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}


/*
Name: getEncodingFormats
Description: 
method of XML RPC server to get Encoding formats (dummy Method, not implemented)
*/
static xmlrpc_value *
getEncodingFormats(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    xmlrpc_int32 state;
   int res = 0;
    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;
  
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

extern int setDecodingFormat(int streamId, char *codecfmt, int width, int ht);
/*
Name: setDecodingFormat
Description: 
method of XML RPC server to set Decoding Format for the imput Stream Id
*/
static xmlrpc_value *
setDecodingFormat(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    
	int res;
	    char *codecId = (char *)malloc(sizeof(char)*255);
	    int width, height,streamId;   		    

	xmlrpc_value * retval;

	xmlrpc_value * arrayP;	
	xmlrpc_decompose_value(env, param_array, "(A)", &arrayP);
	
	size_t size = xmlrpc_array_size(env, arrayP);
	if (env->fault_occurred)
		retval = NULL;
	else 
	{

		for (size_t i = 0; i < size && !env->fault_occurred; ++i) 
		{
			xmlrpc_value * strctP;
			strctP = xmlrpc_array_get_item(env, arrayP, i);
			if (!env->fault_occurred) 
			{
				if(i == 0)
				{
					const char * str1;
					    size_t str1_len;
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
		
					//xmlrpc_int32 curly;
					//xmlrpc_decompose_value(env, strctP, "(s)",&ip[0]);
					strcpy(codecId, str1);
				}
				else
				{
						const char * str1;
					    size_t str1_len;
				
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
					
					if(i == 1)
						width = atoi(str1);
					
					else if(i == 2)
						height = atoi(str1);

					else if(i == 3)
						streamId = atoi(str1);
			
			
				}
			}
		}
	}
        xmlrpc_DECREF(arrayP);
	res = setDecodingFormat(streamId, codecId, width, height);
	if (env->fault_occurred)
        	return NULL;

    
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

extern int setCameraResolution(int ht, int width);
/*
Name: setCameraProps
Description: 
method of XML RPC server to set Camera Properties for the camera device number specified
*/
static xmlrpc_value *
setCameraProps(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    xmlrpc_int32 ht, width;
	int res;

    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(ii)", &width, &ht);
    if (env->fault_occurred)
        return NULL;

	res =  setCameraResolution((int)ht, (int)width);   
	/* Return our result. */
	return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}


extern int setEncodingFormat(int streamId,char *codecfmt, int width, int ht, int bitrate);
/*
Name: setEncodingFormats
Description: 
method of XML RPC server to set Encoding Formats for the input Stream Id
*/
static xmlrpc_value *
setEncodingFormats(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {


	int res;
	char *codecId = (char *)malloc(sizeof(char)*255);
	int width, height, bitrate, streamId;   		    

	xmlrpc_value * retval;

	xmlrpc_value * arrayP;	
	
	xmlrpc_decompose_value(env, param_array, "(A)", &arrayP);

	size_t size = xmlrpc_array_size(env, arrayP);
	if (env->fault_occurred)
		retval = NULL;
	else 
	{

		for (size_t i = 0; i < size && !env->fault_occurred; ++i) 
		{
			xmlrpc_value * strctP;
			strctP = xmlrpc_array_get_item(env, arrayP, i);
			if (!env->fault_occurred) 
			{
				if(i == 0)
				{
					const char * str1;
					    size_t str1_len;
					xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
	
					//xmlrpc_int32 curly;
					//xmlrpc_decompose_value(env, strctP, "(s)",&ip[0]);
					strcpy(codecId, str1);
				}
				else
				{
					const char * str1;
					size_t str1_len;
					xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
				
					if(i == 1)
						width = atoi(str1);
				
					else if(i == 2)
						height = atoi(str1);

					else if(i == 3)
						bitrate = atoi(str1);
					else if(i == 4)
						streamId = atoi(str1);
					
		
				}
			}
		}
	}
	xmlrpc_DECREF(arrayP);

	res = setEncodingFormat(streamId, codecId, width, height, bitrate);
	if (env->fault_occurred)
		return NULL;

	/* Return our result. */
	return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

/*
Name: startLocalPlayback
Description: 
method of XML RPC server to start Local Playback
*/

static xmlrpc_value *
startLocalPlayback(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) 
{

	xmlrpc_int32 state;

    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;

	int res = 0;   
    
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}


/*
Name: StartStreamingRTP
Description: 
method of XML RPC server to start Streaming (sending) video RTP packets
*/
static xmlrpc_value *
StartStreamingRTP(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

    char *ipaddress = (char *)malloc(sizeof(char)*255);
    int port;
    int streamId;	   		    

	xmlrpc_value * retval;
	xmlrpc_value * arrayP;	
	
	xmlrpc_decompose_value(env, param_array, "(A)", &arrayP);
	
	size_t size = xmlrpc_array_size(env, arrayP);
	if (env->fault_occurred)
		retval = NULL;
	else 
	{

		for (size_t i = 0; i < size && !env->fault_occurred; ++i) 
		{
			xmlrpc_value * strctP;
			strctP = xmlrpc_array_get_item(env, arrayP, i);
			if (!env->fault_occurred) 
			{
				if(i == 0)
				{
					const char * str1;
					    size_t str1_len;
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
		
					//xmlrpc_int32 curly;
					//xmlrpc_decompose_value(env, strctP, "(s)",&ip[0]);
					strcpy(ipaddress, str1);
				}
				else if(i == 1)
				{
						const char * str1;
					    size_t str1_len;
				
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
					port = atoi(str1);

					cout << port <<endl;
				
				}
				else if(i == 2)
				{
					const char * str1;
					size_t str1_len;
				
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
					streamId = atoi(str1);

					cout << streamId <<endl;
				
				}
			}
		}
	}
        xmlrpc_DECREF(arrayP);
	int res = 0;
	res = starting(0, ipaddress, port, streamId);
	free(ipaddress);
	/* Return our result. */
    	return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

/*
Name: StartReceivingRTP
Description: 
method of XML RPC server to start receiving video RTP packets
*/
static xmlrpc_value *
StartReceivingRTP(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) 
{

    char ipaddress[255];
    int port;   		    
    int streamId;	
	xmlrpc_value * retval;
	cout << "ReceivingRTP" << endl;
	xmlrpc_value * arrayP;	
	xmlrpc_decompose_value(env, param_array, "(A)", &arrayP);
	
	size_t size = xmlrpc_array_size(env, arrayP);
	if (env->fault_occurred)
		retval = NULL;
	else 
	{

		for (size_t i = 0; i < size && !env->fault_occurred; ++i) 
		{
			xmlrpc_value * strctP;
			strctP = xmlrpc_array_get_item(env, arrayP, i);
			if (!env->fault_occurred) 
			{
				if(i == 0)
				{
					const char * str1;
					    size_t str1_len;
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
		
					//xmlrpc_int32 curly;
					//xmlrpc_decompose_value(env, strctP, "(s)",&ip[0]);
					strcpy(ipaddress, str1);
				}
				else if(i == 1)
				{
						const char * str1;
					    size_t str1_len;
				
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
					port = atoi(str1);

					cout << port <<endl;
				
				}
				else if(i == 2)
				{
					const char * str1;
					size_t str1_len;
				
				        xmlrpc_read_string_lp(env, strctP, &str1_len, &str1);				
					streamId = atoi(str1);

					cout << streamId <<endl;
				
				}
			}
		}
	}
        xmlrpc_DECREF(arrayP);

	int res;
	/*if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER))
		{
		  fprintf(stderr, "Could not initialize SDL - %s\n", SDL_GetError());
		  exit(1);
		}
	*/ 	
	res = starting( 1, ipaddress, port,streamId  );	
  
    /* Sometimes, make it look hard (so client can see what it's like
       to do an RPC that takes a while).
    */
   // int res = startRecv(-2);
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

/*
Name: StopSend
Description: 
method of XML RPC server to stop sending video RTP packets
*/
static xmlrpc_value *
StopSend(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

xmlrpc_int32 state;
    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;

    int res = stopLocal((int)(state));
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}

/*
Name: StopRecv
Description: 
method of XML RPC server to stop receiving video RTP packets
*/
static xmlrpc_value *
StopRecv(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) {

xmlrpc_int32 state;
    /* Parse our argument array. */
    xmlrpc_decompose_value(env, param_array, "(i)", &state);
    if (env->fault_occurred)
        return NULL;

    int res = stopRecv((int)(state));
    /* Return our result. */
    return xmlrpc_build_value(env, "i", (xmlrpc_int32)(res));
}


static xmlrpc_value *
QueryInterface(xmlrpc_env *   const env, 
           xmlrpc_value * const param_array, 
           void *         const user_data ATTR_UNUSED) 
{

	/* Sometimes, make it look hard (so client can see what it's like
	to do an RPC that takes a while).
	*/
	xmlrpc_value *arr = xmlrpc_array_new(env);
	xmlrpc_value *v1 = xmlrpc_build_value(env, "s", "QueryInterface");
	xmlrpc_value *v2 = xmlrpc_build_value(env, "s", "getCameraProps (NA)");	
	xmlrpc_value *v3= xmlrpc_build_value(env, "s", "getEncodingFormats (NA)");	
	xmlrpc_value *v4 = xmlrpc_build_value(env, "s", "setCameraProps");	
	xmlrpc_value *v5 = xmlrpc_build_value(env, "s", "setEncodingFormats");
	xmlrpc_value *v6 = xmlrpc_build_value(env, "s", "setDecodingFormat");	
	xmlrpc_value *v7 = xmlrpc_build_value(env, "s", "startLocalPlayback (NA)");	
	xmlrpc_value *v8 = xmlrpc_build_value(env, "s", "StartStreamingRTP");	
	xmlrpc_value *v9 = xmlrpc_build_value(env, "s", "StartReceivingRTP");	
	xmlrpc_value *v10 = xmlrpc_build_value(env, "s", "StopSend");	
	xmlrpc_value *v11 = xmlrpc_build_value(env, "s", "StopRecv");
	xmlrpc_value *v12 = xmlrpc_build_value(env, "s", "CreateSendStream");
	xmlrpc_value *v13 = xmlrpc_build_value(env, "s", "CreateRecvStream");	


	xmlrpc_array_append_item (env,arr,v1);
	xmlrpc_array_append_item (env,arr,v2);
	xmlrpc_array_append_item (env,arr,v3);
	xmlrpc_array_append_item (env,arr,v4);
	xmlrpc_array_append_item (env,arr,v5);
	xmlrpc_array_append_item (env,arr,v6);
	xmlrpc_array_append_item (env,arr,v7);
	xmlrpc_array_append_item (env,arr,v8);
	xmlrpc_array_append_item (env,arr,v9);
	xmlrpc_array_append_item (env,arr,v10);
	xmlrpc_array_append_item (env,arr,v11);
	xmlrpc_array_append_item (env,arr,v12);
	xmlrpc_array_append_item (env,arr,v13);
	
	return arr;

}

int 
main(int           const argc, 
     const char ** const argv) 
{

	xmlrpc_server_abyss_parms serverparm;
	xmlrpc_registry * registryP;
	xmlrpc_env env;

	if (argc-1 != 1) {
	fprintf(stderr, "You must specify 1 argument:  The TCP port "
		"number on which the server will accept connections "
		"for RPCs.  You specified %d arguments.\n",  argc-1);
	exit(1);
	}

	init_all();
	xmlrpc_env_init(&env);

	registryP = xmlrpc_registry_new(&env);


	xmlrpc_registry_add_method(
	&env, registryP, NULL, "QueryInterface", &QueryInterface, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "getCameraProps", &getCameraProps, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "getEncodingFormats", &getEncodingFormats, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "setCameraProps", &setCameraProps, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "setEncodingFormats", &setEncodingFormats, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "setDecodingFormat", &setDecodingFormat, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "startLocalPlayback", &startLocalPlayback, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "StartStreamingRTP", &StartStreamingRTP, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "StartReceivingRTP", &StartReceivingRTP, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "StopSend", &StopSend, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "StopRecv", &StopRecv, NULL);	
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "CreateSendStream", &CreateSendStream, NULL);
	xmlrpc_registry_add_method(
	&env, registryP, NULL, "CreateRecvStream", &CreateRecvStream, NULL);


	/* In the modern form of the Abyss API, we supply parameters in memory
	like a normal API.  We select the modern form by setting
	config_file_name to NULL: 
	*/
	serverparm.config_file_name = NULL;
	serverparm.registryP        = registryP;
	serverparm.port_number      = atoi(argv[1]);
	serverparm.log_file_name    = "/tmp/xmlrpc_log";

	printf("Running XML-RPC server...\n");
	xmlrpc_server_abyss(&env, &serverparm, XMLRPC_APSIZE(log_file_name));

	/* xmlrpc_server_abyss() never returns */

	return 0;
}
